@@ -18,13 +18,13 @@ from ..autogenerate import compare
 from ..util.compat import string_types
 from ..util.sqla_compat import _is_mariadb
 from ..util.sqla_compat import _is_type_bound
-from ..util.sqla_compat import sqla_100
 
 
 class MySQLImpl(DefaultImpl):
     __dialect__ = "mysql"
 
     transactional_ddl = False
+    type_synonyms = DefaultImpl.type_synonyms + ({"BOOL", "TINYINT"},)
 
     def alter_column(
         self,
@@ -65,6 +65,9 @@ class MySQLImpl(DefaultImpl):
                     autoincrement=autoincrement
                     if autoincrement is not None
                     else existing_autoincrement,
+                    comment=comment
+                    if comment is not False
+                    else existing_comment,
                 )
             )
         elif (
@@ -212,64 +215,6 @@ class MySQLImpl(DefaultImpl):
             if idx.name in removed:
                 metadata_indexes.remove(idx)
 
-        if not sqla_100:
-            self._legacy_correct_for_dupe_uq_uix(
-                conn_unique_constraints,
-                conn_indexes,
-                metadata_unique_constraints,
-                metadata_indexes,
-            )
-
-    def _legacy_correct_for_dupe_uq_uix(
-        self,
-        conn_unique_constraints,
-        conn_indexes,
-        metadata_unique_constraints,
-        metadata_indexes,
-    ):
-
-        # then dedupe unique indexes vs. constraints, since MySQL
-        # doesn't really have unique constraints as a separate construct.
-        # but look in the metadata and try to maintain constructs
-        # that already seem to be defined one way or the other
-        # on that side.  See #276
-        metadata_uq_names = set(
-            [
-                cons.name
-                for cons in metadata_unique_constraints
-                if cons.name is not None
-            ]
-        )
-
-        unnamed_metadata_uqs = set(
-            [
-                compare._uq_constraint_sig(cons).sig
-                for cons in metadata_unique_constraints
-                if cons.name is None
-            ]
-        )
-
-        metadata_ix_names = set(
-            [cons.name for cons in metadata_indexes if cons.unique]
-        )
-        conn_uq_names = dict(
-            (cons.name, cons) for cons in conn_unique_constraints
-        )
-        conn_ix_names = dict(
-            (cons.name, cons) for cons in conn_indexes if cons.unique
-        )
-
-        for overlap in set(conn_uq_names).intersection(conn_ix_names):
-            if overlap not in metadata_uq_names:
-                if (
-                    compare._uq_constraint_sig(conn_uq_names[overlap]).sig
-                    not in unnamed_metadata_uqs
-                ):
-
-                    conn_unique_constraints.discard(conn_uq_names[overlap])
-            elif overlap not in metadata_ix_names:
-                conn_indexes.discard(conn_ix_names[overlap])
-
     def correct_for_autogen_foreignkeys(self, conn_fks, metadata_fks):
         conn_fk_by_sig = dict(
             (compare._fk_constraint_sig(fk).sig, fk) for fk in conn_fks
