@@ -2,16 +2,22 @@ import sys
 
 from sqlalchemy import BIGINT
 from sqlalchemy import BigInteger
+from sqlalchemy import Boolean
 from sqlalchemy import CHAR
 from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
+from sqlalchemy import DATE
 from sqlalchemy import DateTime
 from sqlalchemy import DECIMAL
+from sqlalchemy import Enum
+from sqlalchemy import FLOAT
 from sqlalchemy import ForeignKey
 from sqlalchemy import ForeignKeyConstraint
 from sqlalchemy import Index
+from sqlalchemy import inspect
 from sqlalchemy import INTEGER
 from sqlalchemy import Integer
+from sqlalchemy import LargeBinary
 from sqlalchemy import MetaData
 from sqlalchemy import Numeric
 from sqlalchemy import PrimaryKeyConstraint
@@ -20,15 +26,18 @@ from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy import Text
 from sqlalchemy import text
+from sqlalchemy import TIMESTAMP
 from sqlalchemy import TypeDecorator
+from sqlalchemy import Unicode
 from sqlalchemy import UniqueConstraint
 from sqlalchemy import VARCHAR
 from sqlalchemy.dialects import sqlite
-from sqlalchemy.engine.reflection import Inspector
 from sqlalchemy.types import NULLTYPE
 from sqlalchemy.types import VARBINARY
 
 from alembic import autogenerate
+from alembic import testing
+from alembic.autogenerate import api
 from alembic.migration import MigrationContext
 from alembic.operations import ops
 from alembic.testing import assert_raises_message
@@ -36,9 +45,12 @@ from alembic.testing import config
 from alembic.testing import eq_
 from alembic.testing import is_
 from alembic.testing import is_not_
+from alembic.testing import mock
 from alembic.testing import TestBase
-from alembic.testing.mock import Mock
+from alembic.testing.env import clear_staging_env
+from alembic.testing.env import staging_env
 from alembic.util import CommandError
+from ._autogen_fixtures import _default_object_filters
 from ._autogen_fixtures import AutogenFixtureTest
 from ._autogen_fixtures import AutogenTest
 
@@ -545,7 +557,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
     def test_dont_barf_on_already_reflected(self):
         from sqlalchemy.util import OrderedSet
 
-        inspector = Inspector.from_engine(self.bind)
+        inspector = inspect(self.bind)
         uo = ops.UpgradeOps(ops=[])
         autogenerate.compare._compare_tables(
             OrderedSet([(None, "extra"), (None, "user")]),
@@ -633,7 +645,8 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
 
 
 class CompareTypeSpecificityTest(TestBase):
-    def _fixture(self):
+    @testing.fixture
+    def impl_fixture(self):
         from alembic.ddl import impl
         from sqlalchemy.engine import default
 
@@ -641,7 +654,7 @@ class CompareTypeSpecificityTest(TestBase):
             default.DefaultDialect(), None, False, True, None, {}
         )
 
-    def test_typedec_to_nonstandard(self):
+    def test_typedec_to_nonstandard(self, impl_fixture):
         class PasswordType(TypeDecorator):
             impl = VARBINARY
 
@@ -655,65 +668,183 @@ class CompareTypeSpecificityTest(TestBase):
                     impl = VARBINARY(self.length)
                 return dialect.type_descriptor(impl)
 
-        impl = self._fixture()
-        impl.compare_type(
+        impl_fixture.compare_type(
             Column("x", sqlite.NUMERIC(50)), Column("x", PasswordType(50))
         )
 
-    def test_string(self):
-        t1 = String(30)
-        t2 = String(40)
-        t3 = VARCHAR(30)
-        t4 = Integer
+    @testing.combinations(
+        (VARCHAR(30), String(30), False),
+        (VARCHAR(30), String(40), True),
+        (VARCHAR(30), Integer(), True),
+        (Text(), String(255), True),
+        # insp + metadata types same number of
+        # args but are different; they're different
+        (DECIMAL(10, 5), DECIMAL(10, 6), True),
+        # insp + metadata types, inspected type
+        # has an additional arg; assume this is additional
+        # default precision on the part of the DB, assume they are
+        # equivalent
+        (DECIMAL(10, 5), DECIMAL(10), False),
+        # insp + metadata types, metadata type
+        # has an additional arg; this can go either way, either the
+        # metadata has extra precision, or the DB doesn't support the
+        # element, go with consider them equivalent for now
+        (DECIMAL(10), DECIMAL(10, 5), False),
+        (DECIMAL(10, 2), Numeric(10), False),
+        (DECIMAL(10, 5), Numeric(10, 5), False),
+        (DECIMAL(10, 5), Numeric(12, 5), True),
+        (DECIMAL(10, 5), DateTime(), True),
+        (Numeric(), Numeric(scale=5), False),
+        (INTEGER(), Integer(), False),
+        (BIGINT(), Integer(), True),
+        (BIGINT(), BigInteger(), False),
+        (BIGINT(), SmallInteger(), True),
+        (INTEGER(), SmallInteger(), True),
+        (Integer(), String(), True),
+        id_="ssa",
+        argnames="inspected_type,metadata_type,expected",
+    )
+    def test_compare_type(
+        self, impl_fixture, inspected_type, metadata_type, expected
+    ):
 
-        impl = self._fixture()
-        is_(impl.compare_type(Column("x", t3), Column("x", t1)), False)
-        is_(impl.compare_type(Column("x", t3), Column("x", t2)), True)
-        is_(impl.compare_type(Column("x", t3), Column("x", t4)), True)
+        is_(
+            impl_fixture.compare_type(
+                Column("x", inspected_type), Column("x", metadata_type)
+            ),
+            expected,
+        )
 
-    def test_numeric(self):
-        t1 = Numeric(10, 5)
-        t2 = Numeric(12, 5)
-        t3 = DECIMAL(10, 5)
-        t4 = DateTime
 
-        impl = self._fixture()
-        is_(impl.compare_type(Column("x", t3), Column("x", t1)), False)
-        is_(impl.compare_type(Column("x", t3), Column("x", t2)), True)
-        is_(impl.compare_type(Column("x", t3), Column("x", t4)), True)
+class CompareMetadataToInspectorTest(TestBase):
+    __backend__ = True
 
-    def test_numeric_noprecision(self):
-        t1 = Numeric()
-        t2 = Numeric(scale=5)
+    @classmethod
+    def _get_bind(cls):
+        return config.db
 
-        impl = self._fixture()
-        is_(impl.compare_type(Column("x", t1), Column("x", t2)), False)
+    configure_opts = {}
 
-    def test_integer(self):
-        t1 = Integer()
-        t2 = SmallInteger()
-        t3 = BIGINT()
-        t4 = String()
-        t5 = INTEGER()
-        t6 = BigInteger()
+    def setUp(self):
+        staging_env()
+        self.bind = self._get_bind()
+        self.m1 = MetaData()
 
-        impl = self._fixture()
-        is_(impl.compare_type(Column("x", t5), Column("x", t1)), False)
-        is_(impl.compare_type(Column("x", t3), Column("x", t1)), True)
-        is_(impl.compare_type(Column("x", t3), Column("x", t6)), False)
-        is_(impl.compare_type(Column("x", t3), Column("x", t2)), True)
-        is_(impl.compare_type(Column("x", t5), Column("x", t2)), True)
-        is_(impl.compare_type(Column("x", t1), Column("x", t4)), True)
+    def tearDown(self):
+        self.m1.drop_all(self.bind)
+        clear_staging_env()
 
-    def test_datetime(self):
-        t1 = DateTime()
-        t2 = DateTime(timezone=False)
-        t3 = DateTime(timezone=True)
+    def _compare_columns(self, cola, colb):
+        Table("sometable", self.m1, Column("col", cola))
+        self.m1.create_all(self.bind)
+        m2 = MetaData()
+        Table("sometable", m2, Column("col", colb))
 
-        impl = self._fixture()
-        is_(impl.compare_type(Column("x", t1), Column("x", t2)), False)
-        is_(impl.compare_type(Column("x", t1), Column("x", t3)), True)
-        is_(impl.compare_type(Column("x", t2), Column("x", t3)), True)
+        ctx_opts = {
+            "compare_type": True,
+            "compare_server_default": True,
+            "target_metadata": m2,
+            "upgrade_token": "upgrades",
+            "downgrade_token": "downgrades",
+            "alembic_module_prefix": "op.",
+            "sqlalchemy_module_prefix": "sa.",
+            "include_object": _default_object_filters,
+        }
+        if self.configure_opts:
+            ctx_opts.update(self.configure_opts)
+        with self.bind.connect() as conn:
+            context = MigrationContext.configure(
+                connection=conn, opts=ctx_opts
+            )
+            autogen_context = api.AutogenContext(context, m2)
+            uo = ops.UpgradeOps(ops=[])
+            autogenerate._produce_net_changes(autogen_context, uo)
+        return bool(uo.as_diffs())
+
+    @testing.combinations(
+        (INTEGER(),),
+        (CHAR(),),
+        (VARCHAR(32),),
+        (Text(),),
+        (FLOAT(),),
+        (Numeric(),),
+        (DECIMAL(),),
+        (TIMESTAMP(),),
+        (DateTime(),),
+        (Boolean(),),
+        (BigInteger(),),
+        (SmallInteger(),),
+        (DATE(),),
+        (String(32),),
+        (LargeBinary(),),
+        (Unicode(32),),
+        (Enum("one", "two", "three", name="the_enum"),),
+    )
+    def test_introspected_columns_match_metadata_columns(self, cola):
+        # this is ensuring false positives aren't generated for types
+        # that have not changed.
+        is_(self._compare_columns(cola, cola), False)
+
+    @testing.combinations(
+        (String(32), VARCHAR(32), False),
+        (VARCHAR(6), String(6), False),
+        (CHAR(), String(1), True),
+        (Text(), VARCHAR(255), True),
+        (Unicode(32), String(32), False, config.requirements.unicode_string),
+        (Unicode(32), VARCHAR(32), False, config.requirements.unicode_string),
+        (VARCHAR(6), VARCHAR(12), True),
+        (VARCHAR(6), String(12), True),
+    )
+    def test_string_comparisons(self, cola, colb, expect_changes):
+        is_(self._compare_columns(cola, colb), expect_changes)
+
+    @testing.combinations(
+        (
+            DateTime(),
+            DateTime(timezone=False),
+            False,
+            config.requirements.datetime_timezone,
+        ),
+        (
+            DateTime(),
+            DateTime(timezone=True),
+            True,
+            config.requirements.datetime_timezone,
+        ),
+        (
+            DateTime(timezone=True),
+            DateTime(timezone=False),
+            True,
+            config.requirements.datetime_timezone,
+        ),
+    )
+    def test_datetime_comparisons(self, cola, colb, expect_changes):
+        is_(self._compare_columns(cola, colb), expect_changes)
+
+    @testing.combinations(
+        (Integer(), Integer(), False),
+        (
+            Integer(),
+            Numeric(8, 0),
+            True,
+            config.requirements.integer_subtype_comparisons,
+        ),
+        (Numeric(8, 0), Numeric(8, 2), True),
+        (
+            BigInteger(),
+            Integer(),
+            True,
+            config.requirements.integer_subtype_comparisons,
+        ),
+        (
+            SmallInteger(),
+            Integer(),
+            True,
+            config.requirements.integer_subtype_comparisons,
+        ),
+    )
+    def test_numeric_comparisons(self, cola, colb, expect_changes):
+        is_(self._compare_columns(cola, colb), expect_changes)
 
 
 class AutogenSystemColTest(AutogenTest, TestBase):
@@ -751,11 +882,6 @@ class AutogenSystemColTest(AutogenTest, TestBase):
 class AutogenerateVariantCompareTest(AutogenTest, TestBase):
     __backend__ = True
 
-    # 1.0.13 and lower fail on Postgresql due to variant / bigserial issue
-    # #3739
-
-    __requires__ = ("sqlalchemy_1014",)
-
     @classmethod
     def _get_db_schema(cls):
         m = MetaData()
@@ -824,7 +950,7 @@ class AutogenerateCustomCompareTypeTest(AutogenTest, TestBase):
         return m
 
     def test_uses_custom_compare_type_function(self):
-        my_compare_type = Mock()
+        my_compare_type = mock.Mock()
         self.context._user_compare_type = my_compare_type
 
         uo = ops.UpgradeOps(ops=[])
@@ -853,7 +979,7 @@ class AutogenerateCustomCompareTypeTest(AutogenTest, TestBase):
         eq_(type(inspected_type), INTEGER)
 
     def test_column_type_not_modified_custom_compare_type_returns_False(self):
-        my_compare_type = Mock()
+        my_compare_type = mock.Mock()
         my_compare_type.return_value = False
         self.context._user_compare_type = my_compare_type
 
@@ -865,7 +991,7 @@ class AutogenerateCustomCompareTypeTest(AutogenTest, TestBase):
         eq_(diffs, [])
 
     def test_column_type_modified_custom_compare_type_returns_True(self):
-        my_compare_type = Mock()
+        my_compare_type = mock.Mock()
         my_compare_type.return_value = True
         self.context._user_compare_type = my_compare_type
 
@@ -1041,8 +1167,8 @@ class AutogenerateDiffOrderTest(AutogenTest, TestBase):
 
     def test_diffs_order(self):
         """
-        Added in order to test that child tables(tables with FKs) are generated
-        before their parent tables
+        Added in order to test that child tables(tables with FKs) are
+        generated before their parent tables
         """
 
         ctx = self.autogen_context
@@ -1544,7 +1670,6 @@ class AutoincrementTest(AutogenFixtureTest, TestBase):
         ops = self._fixture(m1, m2, return_ops=True)
         assert "autoincrement" not in ops.ops[0].ops[0].kw
 
-    @config.requirements.fail_before_sqla_110
     def test_alter_column_autoincrement_nonpk_explicit_true(self):
         m1 = MetaData()
         m2 = MetaData()
