@@ -48,6 +48,7 @@ from alembic.testing.env import write_script
 from alembic.testing.fixtures import capture_context_buffer
 from alembic.testing.fixtures import op_fixture
 from alembic.testing.fixtures import TestBase
+from alembic.util import sqla_compat
 
 
 class PostgresqlOpTest(TestBase):
@@ -177,7 +178,7 @@ class PostgresqlOpTest(TestBase):
 
         context.assert_(
             "ALTER TABLE foo.t ALTER COLUMN c SET NOT NULL",
-            "COMMENT ON COLUMN t.c IS 'This is a column comment'",
+            "COMMENT ON COLUMN foo.t.c IS 'This is a column comment'",
         )
 
     @config.requirements.comments_api
@@ -191,7 +192,24 @@ class PostgresqlOpTest(TestBase):
             comment="This is a column comment",
         )
 
-        context.assert_("COMMENT ON COLUMN t.c IS 'This is a column comment'")
+        context.assert_(
+            "COMMENT ON COLUMN foo.t.c IS 'This is a column comment'"
+        )
+
+    @config.requirements.comments_api
+    def test_alter_column_add_comment_table_and_column_quoting(self):
+        context = op_fixture("postgresql")
+        op.alter_column(
+            "T",
+            "C",
+            existing_type=Boolean(),
+            schema="foo",
+            comment="This is a column comment",
+        )
+
+        context.assert_(
+            'COMMENT ON COLUMN foo."T"."C" IS \'This is a column comment\''
+        )
 
     @config.requirements.comments_api
     def test_alter_column_add_comment_quoting(self):
@@ -205,7 +223,7 @@ class PostgresqlOpTest(TestBase):
         )
 
         context.assert_(
-            "COMMENT ON COLUMN t.c IS 'This is a column ''comment'''"
+            "COMMENT ON COLUMN foo.t.c IS 'This is a column ''comment'''"
         )
 
     @config.requirements.comments_api
@@ -220,7 +238,7 @@ class PostgresqlOpTest(TestBase):
             existing_comment="This is a column comment",
         )
 
-        context.assert_("COMMENT ON COLUMN t.c IS NULL")
+        context.assert_("COMMENT ON COLUMN foo.t.c IS NULL")
 
     @config.requirements.comments_api
     def test_create_table_with_comment(self):
@@ -268,6 +286,18 @@ class PostgresqlOpTest(TestBase):
         op.drop_table_comment("t2", existing_comment="t2 table", schema="foo")
         context.assert_("COMMENT ON TABLE foo.t2 IS NULL")
 
+    @config.requirements.computed_columns_api
+    def test_add_column_computed(self):
+        context = op_fixture("postgresql")
+        op.add_column(
+            "t1",
+            Column("some_column", Integer, sqla_compat.Computed("foo * 5")),
+        )
+        context.assert_(
+            "ALTER TABLE t1 ADD COLUMN some_column "
+            "INTEGER GENERATED ALWAYS AS (foo * 5) STORED"
+        )
+
 
 class PGAutocommitBlockTest(TestBase):
     __only_on__ = "postgresql"
